#include "windows.h"
#include "commdlg.h"
#include "stdlib.h"
#include "resource.h"

// ============================================================================

LRESULT WINAPI KeyWndProc( HWND, UINT, WPARAM, LPARAM );
BOOL WINAPI SetupDlgProc( HWND, UINT, WPARAM, LPARAM );
BOOL WINAPI AboutDlgProc( HWND, UINT, WPARAM, LPARAM );
BOOL ModifyTrayIcon( HWND, UINT, UINT, DWORD, int );
BOOL CenterWindow( HWND );
BOOL ListAllApps( HWND );
BOOL SaveAllApps( HWND );
BOOL IsValidKey( void );
BOOL FileBrowser( HWND );

// ============================================================================

HINSTANCE hInst;
HWND hMainWnd;
TCHAR szKeys[ 128 ];
int iKeyLen;
BOOL bInDlg;
TCHAR *szAPPNAME = L"MGQKeys";
TCHAR *szREGKEY = L"Software\\MarkG\\MGQKeys";

// ============================================================================

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrev, LPWSTR lpCmd,
					  int nShow )
{
	WNDCLASS wc;
	MSG msg;


	hMainWnd = FindWindow( szAPPNAME, NULL );
	if( hMainWnd )
	{
		PostMessage( hMainWnd, WM_COMMAND, 1000, 0 );
		return( FALSE );
	}
		
	memset( &wc, 0, sizeof(WNDCLASS) );
	wc.hInstance = hInstance;
	wc.lpszClassName = szAPPNAME;
	wc.lpfnWndProc = KeyWndProc;
	if( ! RegisterClass( &wc ) )
		return( FALSE );

	hInst = hInstance;
	hMainWnd = CreateWindow( szAPPNAME, szAPPNAME, WS_POPUP, 0, 0, 1, 1,
							 NULL, NULL, hInstance, NULL );
	if( ! hMainWnd )
		return( FALSE );

	ShowWindow( hMainWnd, nShow );
	UpdateWindow( hMainWnd );

	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( FALSE );
}

// ============================================================================

LRESULT WINAPI KeyWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
		case WM_CREATE:
			bInDlg = FALSE;
			ModifyTrayIcon( hWnd, WM_COMMAND, 1000, NIM_ADD, IDI_ICON );
			wcscpy( szKeys, L"0123456789ABDFGJLMNOPQSTUVWXYZ" );
			iKeyLen = wcslen( szKeys );
			SetTimer( hWnd, 1, 50, NULL );
			break;


		case WM_DESTROY:
			KillTimer( hWnd, 1 );
			ModifyTrayIcon( hWnd, WM_COMMAND, 1000, NIM_DELETE, IDI_ICON );
			PostQuitMessage( 0 );
			break;


		case WM_TIMER:
		{
			if( GetAsyncKeyState( VK_LWIN ) < 0 )
			{
				KillTimer( hWnd, 1 );

				if( IsValidKey() )
					Sleep( 2000 );
				
				SetTimer( hWnd, 1, 50, NULL );
			}
			break;
		}


		case WM_COMMAND:
			if( wParam == 1000 && ! bInDlg )
			{
				BOOL bQuit;

				bInDlg = TRUE;
				bQuit = DialogBox( hInst, MAKEINTRESOURCE( IDD_DIALOG ), hWnd,
						           SetupDlgProc );
				bInDlg = FALSE;
				if( ! bQuit )
					DestroyWindow( hWnd );
				break;
			}
			break;


		default:
			return( DefWindowProc( hWnd, msg, wParam, lParam ) );
	}
	return( FALSE );
}

// ============================================================================

BOOL ModifyTrayIcon( HWND hWnd, UINT msg, UINT uID, DWORD dwType, int iIcon )
{
	NOTIFYICONDATA nid;


	memset( &nid, 0, sizeof(NOTIFYICONDATA) );
	nid.cbSize = sizeof(NOTIFYICONDATA);
	nid.hWnd = hWnd;
	nid.uID = uID;
	nid.uFlags = NIF_ICON | NIF_MESSAGE;
	nid.uCallbackMessage = msg;
	if( dwType == NIM_ADD )
		nid.hIcon = LoadImage( hInst, MAKEINTRESOURCE( iIcon ),
							   IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR );
	Shell_NotifyIcon( dwType, &nid );
	return( TRUE );
}

// ============================================================================

BOOL WINAPI SetupDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG:
		{
			int i;

			i = 32;
			SendDlgItemMessage( hDlg, IDC_HEADER, LB_SETTABSTOPS, 1,
								(LPARAM)&i );
			SendDlgItemMessage( hDlg, IDC_APPLIST, LB_SETTABSTOPS, 1,
								(LPARAM)&i );
			SendDlgItemMessage( hDlg, IDC_HEADER, LB_ADDSTRING, 0,
								(LPARAM)L"Key\tProgram" );
			SendDlgItemMessage( hDlg, IDC_HEADER, LB_SETCURSEL, 0, 0 );
			ListAllApps( GetDlgItem( hDlg, IDC_APPLIST ) );
			SendDlgItemMessage( hDlg, IDC_APPLIST, LB_SETCURSEL, 0, 0 );
			CenterWindow( hDlg );
			SetForegroundWindow( hDlg );
			return( TRUE );
		}


		case WM_COMMAND:
		{
			if( wParam == IDC_QUIT )
			{
				EndDialog( hDlg, FALSE );
				return( TRUE );
			}

			if( wParam == IDCANCEL )
			{
				SaveAllApps( GetDlgItem( hDlg, IDC_APPLIST ) );
				EndDialog( hDlg, TRUE );
				return( TRUE );
			}

			if( wParam == IDC_ABOUT )
			{
				DialogBox( hInst, MAKEINTRESOURCE( IDD_ABOUT ), hDlg,
						   AboutDlgProc );
				break;
			}

			if( wParam == IDOK )
			{
				int iItem;
				TCHAR szPath[ 512 ], *pszStr;


				iItem = SendDlgItemMessage( hDlg, IDC_APPLIST, LB_GETCURSEL,
											0, 0 );
				if( iItem == LB_ERR )
					break;

				SendDlgItemMessage( hDlg, IDC_APPLIST, LB_GETTEXT, iItem,
									(LPARAM)szPath );
				pszStr = wcschr( szPath, L'\t' );
				if( pszStr )
				{
					++pszStr;
					GetDlgItemText( hDlg, IDC_PROGRAM, pszStr, 509 );
					SendDlgItemMessage( hDlg, IDC_APPLIST, LB_DELETESTRING,
										iItem, 0 );
					iItem = SendDlgItemMessage( hDlg, IDC_APPLIST,
												LB_INSERTSTRING,
												iItem, (LPARAM)szPath );
					SendDlgItemMessage( hDlg, IDC_APPLIST, LB_SETCURSEL,
										iItem, 0 );
				}
				break;
			}

			if( HIWORD( wParam ) == LBN_SELCHANGE &&
				LOWORD( wParam ) == IDC_APPLIST )
			{
				int iItem;
				TCHAR szName[ 512 ], *pszStr;


				iItem = SendDlgItemMessage( hDlg, IDC_APPLIST, LB_GETCURSEL,
											0, 0 );
				if( iItem == LB_ERR )
					break;
				SendDlgItemMessage( hDlg, IDC_APPLIST, LB_GETTEXT, iItem,
									(LPARAM)szName );
				pszStr = wcschr( szName, L'\t' );
				if( pszStr )
				{
					++pszStr;
					SetDlgItemText( hDlg, IDC_PROGRAM, pszStr );
				}
				break;
			}

			if( wParam == IDC_BROWSE )
			{
				FileBrowser( hDlg );
				break;
			}
			break;
		}
	}
	return( FALSE );
}

// ============================================================================

BOOL CenterWindow( HWND hWnd )
{
	int iXsize, iYsize;
	RECT Rect;

	iXsize = GetSystemMetrics( SM_CXSCREEN );
	iYsize = GetSystemMetrics( SM_CYSCREEN );
	GetWindowRect( hWnd, &Rect );
	Rect.right = Rect.right - Rect.left;
	Rect.bottom = Rect.bottom - Rect.top;
	Rect.left = ( iXsize - Rect.right ) / 2;
	Rect.top = ( iYsize - Rect.bottom ) / 2;
	SetWindowPos( hWnd, NULL, Rect.left, Rect.top, 0, 0,
				  SWP_NOSIZE | SWP_NOZORDER );
	return( TRUE );
}

// ============================================================================

BOOL WINAPI AboutDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG:
			CenterWindow( hDlg );
			return( TRUE );


		case WM_COMMAND:
			if( wParam == IDOK || wParam == IDCANCEL )
			{
				EndDialog( hDlg, TRUE );
				return( TRUE );
			}
			break;
	}
	return( FALSE );
}

// ============================================================================

BOOL ListAllApps( HWND hListbox )
{
	LONG lResult;
	HKEY hKey;
	int iIndex;
	DWORD dwDataSize, dwType;
	TCHAR szName[ 4 ], szApp[ 512 ], *pszStr;


	lResult = RegCreateKeyEx( HKEY_CURRENT_USER, szREGKEY,
							  0, NULL, 0, 0, NULL, &hKey, &dwDataSize );
	if( lResult != ERROR_SUCCESS )
		return( FALSE );

	szApp[ 0 ] = L' ';
	szApp[ 2 ] = L'\t';
	szName[ 1 ] = L'\0';
	pszStr = szApp + 3;
	for( iIndex = 0; iIndex < iKeyLen; ++iIndex )
	{
		szApp[ 1 ] = szName[ 0 ] = szKeys[ iIndex ];
		*pszStr = L'\0';
		dwDataSize = 509 * sizeof(TCHAR);
		lResult = RegQueryValueEx( hKey, szName, NULL, &dwType,
								   (LPBYTE)pszStr, &dwDataSize );
		SendMessage( hListbox, LB_ADDSTRING, 0, (LPARAM)szApp );
	}
	RegCloseKey( hKey );
	return( TRUE );
}

// ============================================================================

BOOL SaveAllApps( HWND hListbox )
{
	LONG lResult;
	HKEY hKey;
	DWORD dwDataSize;
	int i;
	TCHAR szStr[ 512 ], *pszStr;


	lResult = RegCreateKeyEx( HKEY_CURRENT_USER, szREGKEY,
							  0, NULL, 0, 0, NULL, &hKey, &dwDataSize );
	if( lResult != ERROR_SUCCESS )
		return( FALSE );

	for( i = 0; i < iKeyLen; i++ )
	{
		SendMessage( hListbox, LB_GETTEXT, i, (LPARAM)szStr );
		pszStr = wcschr( szStr, L'\t' );
		if( *pszStr )
		{
			*pszStr = L'\0';
			++pszStr;
			if( wcslen( pszStr ) )
				RegSetValueEx( hKey, szStr + 1, 0, REG_SZ, (LPBYTE)pszStr,
							   ( wcslen( pszStr ) + 1 ) * sizeof(TCHAR) );
			else
				RegDeleteValue( hKey, szStr + 1 );
		}
	}
	RegCloseKey( hKey );
	return( TRUE );
}

// ============================================================================

BOOL IsValidKey( void )
{
	int i;
	LONG lResult;
	HKEY hKey;
	DWORD dwData, dwType;
	TCHAR szName[ 4 ], szStr[ 512 ], *pszStr;
	PROCESS_INFORMATION pi;


	for( i = 0; i < iKeyLen; i++ )
	{
		if( GetAsyncKeyState( 0x1B ) < 0 )
		{
			MessageBeep( 0xFFFFFFFF );
			PostMessage( hMainWnd, WM_COMMAND, 1000, 0 );
			return( FALSE );
		}
		if( GetAsyncKeyState( (int)szKeys[ i ] ) < 0 )
		{
			lResult = RegCreateKeyEx( HKEY_CURRENT_USER, szREGKEY,
									  0, NULL, 0, 0, NULL, &hKey,
									  &dwData );
			if( lResult == ERROR_SUCCESS )
			{
				szName[ 0 ] = szKeys[ i ];
				szName[ 1 ] = L'\0';
				dwData = sizeof(TCHAR) * 512;
				lResult = RegQueryValueEx( hKey, szName, NULL, &dwType,
										   (LPBYTE)szStr, &dwData );
				if( lResult == ERROR_SUCCESS )
				{
					pszStr = wcschr( szStr, L'~' );
					if( pszStr )
					{
						*pszStr = L'\0';
						++pszStr;
					}

					MessageBeep( 0xFFFFFFFF );
					if( CreateProcess( szStr, pszStr, NULL, NULL, FALSE, 0,
									   NULL, NULL, NULL, &pi ) )
					{
						CloseHandle( pi.hProcess );
						CloseHandle( pi.hThread );
					}
					RegCloseKey( hKey );
					return( TRUE );
				}
				RegCloseKey( hKey );
			}
		}
	}
	return( FALSE );
}

// ============================================================================

BOOL FileBrowser( HWND hDlg )
{
	OPENFILENAME of;
	TCHAR szPath[ 512 ];

	memset( &of, 0, sizeof(OPENFILENAME) );
	szPath[ 0 ] = L'\0';

	of.lStructSize = sizeof(OPENFILENAME);
	of.hwndOwner = hDlg;
	of.lpstrFilter = L"Executables\0*.exe\0All Files\0*.*\0\0";
	of.lpstrFile = szPath;
	of.nMaxFile = 512;
	of.Flags = OFN_FILEMUSTEXIST;

	if( ! GetOpenFileName( &of ) )
		return( FALSE );

	SetDlgItemText( hDlg, IDC_PROGRAM, szPath );
	return( TRUE );
}

// ============================================================================





